#include <SPI.h>
#include <Ethernet.h>
#include "MgsModbus.h"

MgsModbus Mb;
int inByte = 0; // incoming serial byte

void setup()
{
  // serial setup
  Serial.begin(9600);

  // ethernet via dhcp
  uint8_t mac[]     = { 0x90, 0xA2, 0xDA, 0x00, 0x51, 0x06 };
  Ethernet.begin(mac);

  // print your local IP address:
  Serial.print("My IP address: ");
  for (byte thisByte = 0; thisByte < 4; thisByte++) {
    // print the value of each byte of the IP address:
    Serial.print("."); 
  }
  Serial.println();
  
  // slave address
  Mb.remSlaveIP = (192,168,200,56);

  // Fill MbData
  Mb.SetBit(0,false);
  Mb.SetBit(1,true);
  Mb.SetBit(2,0);
  Mb.SetBit(3,1);
  Mb.SetBit(4,false);
  Mb.SetBit(5,false);
  Mb.SetBit(6,true);
  Mb.SetBit(7,true);
  Mb.SetBit(8,false);
  Mb.SetBit(9,false);
  Mb.SetBit(10,false);
  Mb.SetBit(11,false);
  Mb.SetBit(12,true);
  Mb.SetBit(13,true);
  Mb.SetBit(0xE,true);
  Mb.SetBit(0xF,true);
  Mb.MbData[1] = 0;
  Mb.MbData[2] = 0xF0F0;
  Mb.MbData[3] = 0;
  Mb.MbData[4] = 0;
  Mb.MbData[5] = 0;
  Mb.MbData[6] = 0;
  Mb.MbData[7] = 0;
  Mb.MbData[0x8] = 0;
  Mb.MbData[0x9] = 0;
  Mb.MbData[0xA] = 0;
  Mb.MbData[0xB] = 0;
  Mb.MbData[0xC] = 0;
  // print MbData
  for (int i=0;i<12;i++) {
    Serial.print("address: "); Serial.print(i); Serial.print("Data: "); Serial.println(Mb.MbData[i]);
  }
  // print menu
  Serial.println("0 - print the first 5 words of the MbData space");
  Serial.println("1 - FC 1 - read the first 5 coils from the slave and store them in the lower byte of MbData[1]");
  Serial.println("2 - FC 2 - read the first 5 discrete inputs from the slave and store them in the higer of the MbData[1]");
  Serial.println("3 - FC 3 - read the first 5 registers from the slave and store them in MbData[3..7");
  Serial.println("4 - FC 4 - read the first 5 input registers from the slave and store them in MbData[8..12]");
  Serial.println("5 - FC 5 - write coil 0 of the slave with the bit valeu ot MbData[0.0]");
  Serial.println("6 - FC 6 - write register 0 of the slave with MbData[2]");
  Serial.println("7 - FC 15 - write 5 coils of the slave starting with coil 0 with GetBit(16..20");
  Serial.println("8 - Fc 16 - weite 5 registers of the slave starting on register 0 with MbData[0..4]");
}

void loop()
{
  if (Serial.available() > 0) {
    // get incoming byte:
    inByte = Serial.read();
    if (inByte == '0') {                                          // print MbData
      for (int i=0;i<12;i++) {
        Serial.print("address: "); Serial.print(i); Serial.print("Data: "); Serial.println(Mb.MbData[i]);
      }
    }  
    if (inByte == '1') {Mb.Req(MB_FC_READ_COILS,               0,5,0x10);} // 1 // ref, count, pos
    if (inByte == '2') {Mb.Req(MB_FC_READ_DISCRETE_INPUT,      0,5,0x18);} // 2
    if (inByte == '3') {Mb.Req(MB_FC_READ_REGISTERS,           0,5,3);} // 3
    if (inByte == '4') {Mb.Req(MB_FC_READ_INPUT_REGISTER,      0,5,8);} // 4
    if (inByte == '5') {Mb.Req(MB_FC_WRITE_COIL,               0,0,0);} // 5 // count can be x
    if (inByte == '6') {Mb.Req(MB_FC_WRITE_REGISTER,           0,0,2);} // 6 // count can be x
    if (inByte == '7') {Mb.Req(MB_FC_WRITE_MULTIPLE_COILS,     0,5,0);} // 15
    if (inByte == '8') {Mb.Req(MB_FC_WRITE_MULTIPLE_REGISTERS, 0,5,0);} // 16
  }
  Mb.MbmRun();
  Mb.MbsRun();
}

